@@ -146,6 +146,11 @@ static int	copy(Tcl_Interp *interp, int, const char *, int, const char *, off_t)
 static int	compare(int, const char *, size_t, int, const char *, size_t);
 static int	create_newfile(Tcl_Interp *interp, const char *, int, struct stat *);
 static int	create_tempfile(const char *, char *, size_t);
+#if defined(UF_IMMUTABLE) && defined(SF_IMMUTABLE)
+static int	handle_existing_file(Tcl_Interp *interp, const char *path, struct stat *sbp);
+#else
+static int	handle_existing_file(Tcl_Interp *interp, const char *path, struct stat *sbp UNUSED);
+#endif
 static int	install(Tcl_Interp *interp, const char *, const char *, u_long, u_int);
 static int	install_dir(Tcl_Interp *interp, char *);
 static u_long	numeric_id(Tcl_Interp *interp, const char *, const char *, int *rval);
@@ -448,7 +453,7 @@ install(Tcl_Interp *interp, const char *from_name, const char *to_name, u_long f
 	int tempcopy, temp_fd = -1, to_fd = -1;
 	char backup[MAXPATHLEN], *p, pathbuf[MAXPATHLEN], tempfile[MAXPATHLEN];
 #if defined(HAVE_CLONEFILE) && defined(CLONE_NOOWNERCOPY)
-	const int tryclone = 1;
+	int tryclone = 1;
 #else
 	const int tryclone = 0;
 #endif
@@ -537,7 +542,12 @@ install(Tcl_Interp *interp, const char *from_name, const char *to_name, u_long f
 	    int done_clone = 0;
 #if defined(HAVE_CLONEFILE) && defined(CLONE_NOOWNERCOPY)
 	    if (tryclone && !devnull && !dostrip) {
-	        done_clone = (clone(from_name, to_name, tempcopy, tempfile, sizeof(tempfile)) == 0);
+	        if (!tempcopy && target && handle_existing_file(interp, to_name, &to_sb) != 0) {
+	            tryclone = 0;
+	        }
+	        if (tryclone) {
+	            done_clone = (clone(from_name, to_name, tempcopy, tempfile, sizeof(tempfile)) == 0);
+	        }
 	    }
 	    if (done_clone &&
 	        (((to_fd = open(tempcopy ? tempfile : to_name, O_RDONLY, 0)) < 0)
@@ -905,55 +915,64 @@ create_tempfile(const char *path, char *temp, size_t tsize)
 	return (mkstemp(temp));
 }
 
+/* unlink or move (backup) an existing file */
+static int
+#if defined(UF_IMMUTABLE) && defined(SF_IMMUTABLE)
+handle_existing_file(Tcl_Interp *interp, const char *path, struct stat *sbp)
+#else
+handle_existing_file(Tcl_Interp *interp, const char *path, struct stat *sbp UNUSED)
+#endif
+{
+    char backup[MAXPATHLEN];
+    /*
+     * Unlink now... avoid ETXTBSY errors later.  Try to turn
+     * off the append/immutable bits -- if we fail, go ahead,
+     * it might work.
+     */
+#if defined(UF_IMMUTABLE) && defined(SF_IMMUTABLE)
+    if (sbp->st_flags & NOCHANGEBITS)
+        (void)chflags(path, sbp->st_flags & ~NOCHANGEBITS);
+#endif
+
+    if (dobackup) {
+        if ((size_t)snprintf(backup, MAXPATHLEN, "%s%s",
+                     path, suffix) != strlen(path) + strlen(suffix)) {
+            char errmsg[255];
+
+            snprintf(errmsg, sizeof errmsg, "%s: Backup filename %s too long",
+                 funcname, path);
+            Tcl_SetResult(interp, errmsg, TCL_VOLATILE);
+            return EINVAL;
+        }
+        (void)snprintf(backup, MAXPATHLEN, "%s%s", path, suffix);
+                    ui_info(interp, "%s: %s -> %s", funcname, path, backup);
+        if (rename(path, backup) < 0) {
+            char errmsg[255];
+            int saved_errno = errno;
+
+            snprintf(errmsg, sizeof errmsg, "%s: Rename: %s to %s, %s",
+                 funcname, path, backup, strerror(errno));
+            Tcl_SetResult(interp, errmsg, TCL_VOLATILE);
+            return saved_errno;
+        }
+    } else if (unlink(path) < 0) {
+        return errno;
+    }
+    return 0;
+}
+
 /*
  * create_newfile --
  *	create a new file, overwriting an existing one if necessary
  */
 static int
-#if defined(UF_IMMUTABLE) && defined(SF_IMMUTABLE)
 create_newfile(Tcl_Interp *interp, const char *path, int target, struct stat *sbp)
-#else
-create_newfile(Tcl_Interp *interp, const char *path, int target, struct stat *sbp UNUSED)
-#endif
 {
-	char backup[MAXPATHLEN];
 	int saved_errno = 0;
 	int newfd;
 
 	if (target) {
-		/*
-		 * Unlink now... avoid ETXTBSY errors later.  Try to turn
-		 * off the append/immutable bits -- if we fail, go ahead,
-		 * it might work.
-		 */
-#if defined(UF_IMMUTABLE) && defined(SF_IMMUTABLE)
-		if (sbp->st_flags & NOCHANGEBITS)
-			(void)chflags(path, sbp->st_flags & ~NOCHANGEBITS);
-#endif
-
-		if (dobackup) {
-			if ((size_t)snprintf(backup, MAXPATHLEN, "%s%s",
-					     path, suffix) != strlen(path) + strlen(suffix)) {
-				char errmsg[255];
-
-				snprintf(errmsg, sizeof errmsg, "%s: Backup filename %s too long",
-					 funcname, path);
-				Tcl_SetResult(interp, errmsg, TCL_VOLATILE);
-				return -1;
-			}
-			(void)snprintf(backup, MAXPATHLEN, "%s%s", path, suffix);
-                        ui_info(interp, "%s: %s -> %s", funcname, path, backup);
-			if (rename(path, backup) < 0) {
-				char errmsg[255];
-
-				snprintf(errmsg, sizeof errmsg, "%s: Rename: %s to %s, %s",
-					 funcname, path, backup, strerror(errno));
-				Tcl_SetResult(interp, errmsg, TCL_VOLATILE);
-				return -1;
-			}
-		} else
-			if (unlink(path) < 0)
-				saved_errno = errno;
+	    saved_errno = handle_existing_file(interp, path, sbp);
 	}
 
 	newfd = open(path, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR);
@@ -1054,7 +1073,7 @@ clone(const char *from_name, const char *to_name,
         mktemp(temp_name);
         to_name = temp_name;
     }
-    return clonefile(from_name, to_name, CLONE_NOFOLLOW|CLONE_NOOWNERCOPY);
+    return clonefile(from_name, to_name, CLONE_NOOWNERCOPY);
 }
 #endif
 
